"""
Webhook 服务层

处理 webhook 通知的接收、存储和 Redis pub/sub 通知
"""
from datetime import datetime, timezone
from typing import Optional, Dict, Any, List
import json
import logging

from sqlalchemy import select, func, desc, update
from sqlalchemy.ext.asyncio import AsyncSession

from jettask.db.models.webhook import Webhook, WebhookStatus
from jettask.schemas.webhook import (
    WebhookReceiveResponse,
    WebhookInfo,
    WebhookListResponse
)

logger = logging.getLogger(__name__)

# Redis channel 前缀
WEBHOOK_CHANNEL_PREFIX = "jettask:webhook:"


class WebhookService:
    """Webhook 服务类"""

    @staticmethod
    def get_channel_name(namespace: str, callback_id: str) -> str:
        """获取 Redis pub/sub channel 名称"""
        return f"{WEBHOOK_CHANNEL_PREFIX}{namespace}:{callback_id}"

    @staticmethod
    async def receive_webhook(
        pg_session: AsyncSession,
        redis_client,
        namespace: str,
        callback_id: str,
        payload: Dict[str, Any]
    ) -> WebhookReceiveResponse:
        """
        接收并处理 webhook 通知

        Args:
            pg_session: PostgreSQL 会话
            redis_client: Redis 客户端
            namespace: 命名空间
            callback_id: 回调 ID
            payload: 原始 webhook 载荷

        Returns:
            WebhookReceiveResponse: 处理结果
        """
        # 1. 存储到数据库
        webhook = Webhook(
            namespace=namespace,
            callback_id=callback_id,
            payload=payload,
            status=WebhookStatus.PENDING,
            received_at=datetime.now(timezone.utc)
        )

        pg_session.add(webhook)
        await pg_session.commit()

        logger.info(f"Webhook 已接收: callback_id={callback_id}, namespace={namespace}")

        # 2. 通过 Redis pub/sub 通知等待的任务
        channel = WebhookService.get_channel_name(namespace, callback_id)
        message = json.dumps({
            "callback_id": callback_id,
            "payload": payload,
            "received_at": datetime.now(timezone.utc).isoformat()
        })

        try:
            await redis_client.publish(channel, message)
            logger.debug(f"Webhook 通知已发布到 channel: {channel}")
        except Exception as e:
            logger.warning(f"发布 webhook 通知失败: {e}")

        # 3. 同时写入 Redis 作为备份（防止 pub/sub 消息丢失）
        # 使用 list 存储，支持同一个 callback_id 的多次回调
        result_key = f"{WEBHOOK_CHANNEL_PREFIX}result:{namespace}:{callback_id}"
        try:
            await redis_client.rpush(result_key, message)
            # 设置 24 小时过期
            await redis_client.expire(result_key, 86400)
        except Exception as e:
            logger.warning(f"写入 webhook 结果到 Redis 失败: {e}")

        return WebhookReceiveResponse(
            success=True,
            message="Webhook received successfully",
            callback_id=callback_id
        )

    @staticmethod
    async def list_webhooks(
        pg_session: AsyncSession,
        namespace: str,
        callback_id: Optional[str] = None,
        status: Optional[WebhookStatus] = None,
        page: int = 1,
        page_size: int = 20
    ) -> WebhookListResponse:
        """
        查询 webhook 列表

        Args:
            pg_session: PostgreSQL 会话
            namespace: 命名空间
            callback_id: 按 callback_id 筛选（可选）
            status: 按状态筛选（可选）
            page: 页码
            page_size: 每页大小

        Returns:
            WebhookListResponse: 分页的 webhook 列表
        """
        # 构建查询
        query = select(Webhook).where(Webhook.namespace == namespace)

        if callback_id:
            query = query.where(Webhook.callback_id == callback_id)
        if status:
            query = query.where(Webhook.status == status)

        # 计算总数
        count_query = select(func.count()).select_from(query.subquery())
        total_result = await pg_session.execute(count_query)
        total = total_result.scalar() or 0

        # 分页和排序
        query = query.order_by(desc(Webhook.received_at))
        query = query.offset((page - 1) * page_size).limit(page_size)

        # 执行查询
        result = await pg_session.execute(query)
        webhooks = result.scalars().all()

        # 转换为响应模型
        data = []
        for webhook in webhooks:
            data.append(WebhookInfo(
                id=webhook.id,
                namespace=webhook.namespace,
                callback_id=webhook.callback_id,
                payload=webhook.payload,
                status=webhook.status,
                received_at=webhook.received_at
            ))

        return WebhookListResponse(
            success=True,
            data=data,
            total=total,
            page=page,
            page_size=page_size
        )

    @staticmethod
    async def get_webhook_by_callback_id(
        pg_session: AsyncSession,
        namespace: str,
        callback_id: str
    ) -> Optional[WebhookInfo]:
        """
        根据 callback_id 获取最新的 webhook

        Args:
            pg_session: PostgreSQL 会话
            namespace: 命名空间
            callback_id: 回调 ID

        Returns:
            WebhookInfo 或 None
        """
        query = select(Webhook).where(
            Webhook.namespace == namespace,
            Webhook.callback_id == callback_id
        ).order_by(desc(Webhook.received_at)).limit(1)

        result = await pg_session.execute(query)
        webhook = result.scalar_one_or_none()

        if not webhook:
            return None

        return WebhookInfo(
            id=webhook.id,
            namespace=webhook.namespace,
            callback_id=webhook.callback_id,
            payload=webhook.payload,
            status=webhook.status,
            received_at=webhook.received_at
        )

    @staticmethod
    async def mark_webhook_success(
        pg_session: AsyncSession,
        namespace: str,
        callback_id: str
    ) -> bool:
        """
        标记 webhook 为已处理成功

        Args:
            pg_session: PostgreSQL 会话
            namespace: 命名空间
            callback_id: 回调 ID

        Returns:
            是否更新成功
        """
        stmt = (
            update(Webhook)
            .where(
                Webhook.namespace == namespace,
                Webhook.callback_id == callback_id,
                Webhook.status == WebhookStatus.PENDING
            )
            .values(status=WebhookStatus.SUCCESS)
        )

        result = await pg_session.execute(stmt)
        await pg_session.commit()

        return result.rowcount > 0



"""
Webhook 工具类

提供给任务使用的 webhook 相关工具函数
"""
import asyncio
import json
import uuid
import logging
from typing import Optional, Dict, Any
from urllib.parse import urljoin

logger = logging.getLogger(__name__)

# Redis key 前缀
WEBHOOK_CHANNEL_PREFIX = "jettask:webhook:"


def generate_callback_id() -> str:
    """
    生成唯一的回调 ID

    Returns:
        str: 格式为 cb_{uuid} 的回调 ID
    """
    return f"cb_{uuid.uuid4().hex[:16]}"


def build_webhook_url(base_url: str, namespace: str, callback_id: str) -> str:
    """
    构建 webhook 回调 URL

    Args:
        base_url: 服务基础 URL，如 https://api.example.com
        namespace: 命名空间
        callback_id: 回调 ID

    Returns:
        str: 完整的 webhook URL

    Example:
        >>> build_webhook_url("https://api.example.com", "default", "cb_abc123")
        'https://api.example.com/api/task/v1/default/webhooks/cb_abc123'
    """
    # 确保 base_url 以 / 结尾
    if not base_url.endswith('/'):
        base_url = base_url + '/'

    path = f"api/task/v1/{namespace}/webhooks/{callback_id}"
    return urljoin(base_url, path)


async def wait_for_webhook(
    redis_client,
    namespace: str,
    callback_id: str,
    timeout: float = 300.0,
    fallback_poll_interval: float = 5.0
) -> Optional[Dict[str, Any]]:
    """
    等待 webhook 回调结果

    使用 Redis pub/sub 作为主要通知机制，同时有一个低频轮询作为兜底，
    防止 pub/sub 连接异常断开导致消息丢失。

    Args:
        redis_client: Redis 客户端
        namespace: 命名空间
        callback_id: 回调 ID
        timeout: 超时时间（秒），默认 300 秒（5分钟）
        fallback_poll_interval: 兜底轮询间隔（秒），默认 5 秒

    Returns:
        Dict[str, Any]: webhook 载荷数据，超时返回 None

    Example:
        ```python
        @app.task(queue="image_gen")
        async def generate_image(prompt: str):
            # 1. 生成回调 ID 和 URL
            callback_id = generate_callback_id()
            webhook_url = build_webhook_url(
                "https://api.example.com",
                "default",
                callback_id
            )

            # 2. 调用第三方 API，传递 webhook URL
            response = await third_party_api.generate(
                prompt=prompt,
                webhook_url=webhook_url
            )

            # 3. 等待 webhook 回调
            result = await wait_for_webhook(
                redis_client=app.redis,
                namespace="default",
                callback_id=callback_id,
                timeout=600  # 10 分钟超时
            )

            if result is None:
                raise TimeoutError("等待第三方回调超时")

            return result
        ```
    """
    result_key = f"{WEBHOOK_CHANNEL_PREFIX}result:{namespace}:{callback_id}"
    channel_name = f"{WEBHOOK_CHANNEL_PREFIX}{namespace}:{callback_id}"

    # 先检查是否已经有结果（处理 webhook 在订阅前就到达的情况）
    existing = await redis_client.lpop(result_key)
    if existing:
        try:
            data = json.loads(existing)
            logger.info(f"找到已存在的 webhook 结果: callback_id={callback_id}")
            return data.get("payload")
        except json.JSONDecodeError:
            logger.warning(f"解析已存在的 webhook 结果失败: {existing}")

    # 创建 pub/sub 订阅
    pubsub = redis_client.pubsub()
    await pubsub.subscribe(channel_name)

    try:
        start_time = asyncio.get_event_loop().time()
        last_poll_time = start_time

        while True:
            # 检查是否超时
            current_time = asyncio.get_event_loop().time()
            elapsed = current_time - start_time
            if elapsed >= timeout:
                logger.warning(f"等待 webhook 超时: callback_id={callback_id}")
                return None

            # 尝试从 pub/sub 获取消息（短超时，快速响应）
            try:
                message = await asyncio.wait_for(
                    pubsub.get_message(ignore_subscribe_messages=True, timeout=0.5),
                    timeout=1.0
                )

                if message and message.get("type") == "message":
                    try:
                        data = json.loads(message["data"])
                        logger.info(f"通过 pub/sub 收到 webhook 结果: callback_id={callback_id}")
                        return data.get("payload")
                    except json.JSONDecodeError:
                        logger.warning(f"解析 pub/sub 消息失败: {message}")

            except asyncio.TimeoutError:
                pass
            except Exception as e:
                # pub/sub 可能断开，记录警告但继续轮询
                logger.warning(f"pub/sub 异常: {e}")

            # 兜底轮询：定期检查 Redis list（低频率）
            if current_time - last_poll_time >= fallback_poll_interval:
                last_poll_time = current_time
                try:
                    result = await redis_client.lpop(result_key)
                    if result:
                        try:
                            data = json.loads(result)
                            logger.info(f"通过兜底轮询收到 webhook 结果: callback_id={callback_id}")
                            return data.get("payload")
                        except json.JSONDecodeError:
                            logger.warning(f"解析 Redis list 结果失败: {result}")
                except Exception as e:
                    logger.warning(f"兜底轮询 Redis 失败: {e}")

            # 短暂休眠避免 CPU 空转
            await asyncio.sleep(0.1)

    finally:
        # 清理订阅
        try:
            await pubsub.unsubscribe(channel_name)
            await pubsub.close()
        except Exception:
            pass


class WebhookAwaiter:
    """
    Webhook 等待器

    提供更友好的 API 来等待 webhook 回调

    Example:
        ```python
        @app.task(queue="image_gen")
        async def generate_image(prompt: str):
            # 使用 WebhookAwaiter
            async with WebhookAwaiter(
                redis_client=app.redis,
                namespace="default",
                base_url="https://api.example.com"
            ) as awaiter:
                # 获取 webhook URL
                webhook_url = awaiter.webhook_url

                # 调用第三方 API
                await third_party_api.generate(prompt=prompt, webhook_url=webhook_url)

                # 等待结果
                result = await awaiter.wait(timeout=600)

            return result
        ```
    """

    def __init__(
        self,
        redis_client,
        namespace: str,
        base_url: str,
        callback_id: Optional[str] = None
    ):
        """
        初始化 WebhookAwaiter

        Args:
            redis_client: Redis 客户端
            namespace: 命名空间
            base_url: 服务基础 URL
            callback_id: 回调 ID（可选，不提供则自动生成）
        """
        self.redis_client = redis_client
        self.namespace = namespace
        self.base_url = base_url
        self.callback_id = callback_id or generate_callback_id()
        self._webhook_url = None

    @property
    def webhook_url(self) -> str:
        """获取 webhook URL"""
        if self._webhook_url is None:
            self._webhook_url = build_webhook_url(
                self.base_url, self.namespace, self.callback_id
            )
        return self._webhook_url

    async def wait(
        self,
        timeout: float = 300.0,
        fallback_poll_interval: float = 5.0
    ) -> Optional[Dict[str, Any]]:
        """
        等待 webhook 回调结果

        Args:
            timeout: 超时时间（秒）
            fallback_poll_interval: 兜底轮询间隔（秒），默认 5 秒

        Returns:
            webhook 载荷数据，超时返回 None
        """
        return await wait_for_webhook(
            redis_client=self.redis_client,
            namespace=self.namespace,
            callback_id=self.callback_id,
            timeout=timeout,
            fallback_poll_interval=fallback_poll_interval
        )

    async def __aenter__(self):
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        # 清理 Redis 中的结果数据
        result_key = f"{WEBHOOK_CHANNEL_PREFIX}result:{self.namespace}:{self.callback_id}"
        try:
            await self.redis_client.delete(result_key)
        except Exception:
            pass
        return False



"""
命名空间服务

提供命名空间的完整管理功能，包括增删改查、激活/停用、统计等
使用 SQLAlchemy ORM 操作数据库
"""
import logging
from typing import List, Optional, Dict, Any
from datetime import datetime
from urllib.parse import urlparse
from sqlalchemy import select, update, func
from sqlalchemy.ext.asyncio import AsyncSession

from jettask.db.models import Namespace
from jettask.schemas import (
    ConfigMode,
    NamespaceCreate,
    NamespaceUpdate,
    NamespaceResponse
)

logger = logging.getLogger(__name__)


class NamespaceService:
    """命名空间服务类 - 提供命名空间的所有管理功能"""

    @staticmethod
    def validate_redis_url(redis_url: str) -> bool:
        """
        验证 Redis URL 格式

        Args:
            redis_url: Redis 连接 URL

        Returns:
            是否为有效的 Redis URL

        示例:
            redis://localhost:6379/0
            redis://:password@localhost:6379/0
            rediss://localhost:6380/0
        """
        try:
            parsed = urlparse(redis_url)
            return parsed.scheme in ['redis', 'rediss']
        except Exception:
            return False

    @staticmethod
    def validate_pg_url(pg_url: str) -> bool:
        """
        验证 PostgreSQL URL 格式

        Args:
            pg_url: PostgreSQL 连接 URL

        Returns:
            是否为有效的 PostgreSQL URL

        支持格式:
            - postgresql://user:password@host:port/database
            - postgresql+asyncpg://user:password@host:port/database
            - postgres://user:password@host:port/database
        """
        try:
            parsed = urlparse(pg_url)
            return parsed.scheme in ['postgresql', 'postgres', 'postgresql+asyncpg']
        except Exception:
            return False

    @staticmethod
    def mask_url_password(url: str) -> str:
        """
        将 URL 中的密码部分替换为星号，用于日志记录

        Args:
            url: 包含密码的 URL

        Returns:
            脱敏后的 URL

        示例:
            输入: redis://:mypassword@localhost:6379/0
            输出: redis://:***@localhost:6379/0
        """
        try:
            parsed = urlparse(url)
            if parsed.password:
                # 构建脱敏后的URL
                if parsed.username:
                    netloc = f"{parsed.username}:***@{parsed.hostname}"
                else:
                    netloc = f":***@{parsed.hostname}"

                if parsed.port:
                    netloc += f":{parsed.port}"

                masked_url = f"{parsed.scheme}://{netloc}{parsed.path}"
                if parsed.query:
                    masked_url += f"?{parsed.query}"
                if parsed.fragment:
                    masked_url += f"#{parsed.fragment}"

                return masked_url
            return url
        except Exception:
            return url

    @staticmethod
    async def get_config_from_nacos(key: str) -> str:
        """
        从 Nacos 获取配置值

        Args:
            key: Nacos 配置键

        Returns:
            配置值（URL 格式的字符串）

        Raises:
            ValueError: 配置键不存在或获取失败
        """
        try:
            from jettask.config.nacos_config import config
            value = config.config.get(key)
            if not value:
                raise ValueError(f"Nacos配置键 '{key}' 不存在或为空")
            return value
        except ImportError:
            raise ValueError("无法加载Nacos配置模块")
        except Exception as e:
            raise ValueError(f"从Nacos获取配置失败: {str(e)}")

    @staticmethod
    def _build_namespace_response(ns: Namespace) -> NamespaceResponse:
        """
        构建命名空间响应对象

        这是一个内部辅助方法，用于将 ORM 模型转换为 API 响应格式。
        根据配置模式（direct/nacos）返回不同的字段：
        - direct 模式：返回完整的连接 URL
        - nacos 模式：返回 nacos_key，不返回真实 URL（安全考虑）

        Args:
            ns: Namespace ORM 模型实例

        Returns:
            命名空间响应对象
        """
        redis_config_dict = ns.redis_config if ns.redis_config else {}
        pg_config_dict = ns.pg_config if ns.pg_config else {}

        # 获取配置模式
        redis_config_mode = redis_config_dict.get('config_mode', 'direct')
        pg_config_mode = pg_config_dict.get('config_mode', 'direct')

        # Redis 配置处理 - 根据模式返回不同内容
        if redis_config_mode == 'nacos':
            # Nacos 模式 - 只返回 nacos_key，不返回真实 URL（保密）
            redis_url = None
            redis_nacos_key = redis_config_dict.get('nacos_key')
            logger.debug(f"命名空间 {ns.name} 使用 Nacos 模式，返回 Redis key: {redis_nacos_key}")
        else:
            # Direct 模式 - 返回真实完整的 URL（客户端需要使用）
            redis_url = redis_config_dict.get('url', '')
            redis_nacos_key = None
            logger.debug(f"命名空间 {ns.name} 使用 Direct 模式，返回真实 Redis URL")

        # PostgreSQL 配置处理 - 根据模式返回不同内容
        if pg_config_mode == 'nacos':
            # Nacos 模式 - 只返回 nacos_key，不返回真实 URL（保密）
            pg_url = None
            pg_nacos_key = pg_config_dict.get('nacos_key')
            logger.debug(f"命名空间 {ns.name} 使用 Nacos 模式，返回 PG key: {pg_nacos_key}")
        else:
            # Direct 模式 - 返回真实完整的 URL（客户端需要使用）
            pg_url = pg_config_dict.get('url')
            pg_nacos_key = None
            if pg_url:
                logger.debug(f"命名空间 {ns.name} 使用 Direct 模式，返回真实 PG URL")

        return NamespaceResponse(
            name=ns.name,
            description=ns.description,
            redis_url=redis_url,
            redis_config_mode=redis_config_mode,
            redis_nacos_key=redis_nacos_key,
            pg_url=pg_url,
            pg_config_mode=pg_config_mode,
            pg_nacos_key=pg_nacos_key,
            connection_url=f"/api/v1/namespaces/{ns.name}",
            version=ns.version or 1,
            enabled=ns.is_active,
            created_at=ns.created_at,
            updated_at=ns.updated_at
        )

    @staticmethod
    async def list_namespaces(
        session: AsyncSession,
        page: int = 1,
        page_size: int = 20,
        is_active: Optional[bool] = None
    ) -> List[NamespaceResponse]:
        """
        列出所有命名空间

        Args:
            session: 数据库会话
            page: 页码（从1开始）
            page_size: 每页数量
            is_active: 是否只返回激活的命名空间，None 表示返回所有

        Returns:
            命名空间列表
        """
        logger.info(f"列出命名空间: page={page}, page_size={page_size}, is_active={is_active}")

        # 构建查询
        stmt = select(Namespace)

        if is_active is not None:
            stmt = stmt.where(Namespace.is_active == is_active)

        stmt = stmt.order_by(Namespace.created_at.desc())
        stmt = stmt.offset((page - 1) * page_size).limit(page_size)

        result = await session.execute(stmt)
        namespaces = result.scalars().all()

        responses = [NamespaceService._build_namespace_response(ns) for ns in namespaces]

        logger.info(f"成功获取 {len(responses)} 个命名空间")
        return responses

    @staticmethod
    async def create_namespace(
        session: AsyncSession,
        namespace: NamespaceCreate
    ) -> NamespaceResponse:
        """
        创建新的命名空间

        Args:
            session: 数据库会话
            namespace: 命名空间创建信息

        Returns:
            创建的命名空间信息

        Raises:
            ValueError: 命名空间已存在、配置无效等
        """
        # 确定 Redis 和 PostgreSQL 的配置模式（支持独立配置和向后兼容）
        redis_config_mode = namespace.redis_config_mode or namespace.config_mode or ConfigMode.DIRECT
        pg_config_mode = namespace.pg_config_mode or namespace.config_mode or ConfigMode.DIRECT

        logger.info(
            f"创建命名空间: {namespace.name}, "
            f"Redis模式: {redis_config_mode}, PG模式: {pg_config_mode}"
        )

        # 检查命名空间是否已存在
        stmt = select(func.count()).select_from(Namespace).where(Namespace.name == namespace.name)
        result = await session.execute(stmt)
        if result.scalar() > 0:
            logger.error(f"命名空间 '{namespace.name}' 已存在")
            raise ValueError(f"命名空间 '{namespace.name}' 已存在")

        # 准备 Redis 配置
        redis_config = {'config_mode': redis_config_mode.value}
        if redis_config_mode == ConfigMode.DIRECT:
            # 直接配置模式
            if not namespace.redis_url:
                raise ValueError("Redis直接配置模式下，redis_url是必需的")
            if namespace.redis_nacos_key:
                raise ValueError("Redis直接配置模式下不应提供redis_nacos_key")

            # 验证URL格式
            if not NamespaceService.validate_redis_url(namespace.redis_url):
                raise ValueError("无效的Redis URL格式")

            redis_config['url'] = namespace.redis_url
            logger.debug(f"Redis URL: {NamespaceService.mask_url_password(namespace.redis_url)}")

        elif redis_config_mode == ConfigMode.NACOS:
            # Nacos配置模式 - 只存储元数据，不连接Nacos获取实际配置
            if not namespace.redis_nacos_key:
                raise ValueError("Redis Nacos配置模式下，redis_nacos_key是必需的")
            if namespace.redis_url:
                raise ValueError("Redis Nacos配置模式下不应提供redis_url")

            # 只存储 nacos_key，不获取实际URL（保证数据安全）
            redis_config['nacos_key'] = namespace.redis_nacos_key
            logger.info(f"Redis Nacos模式：存储配置键 '{namespace.redis_nacos_key}'")

        # 准备 PostgreSQL 配置
        pg_config = {'config_mode': pg_config_mode.value}
        if pg_config_mode == ConfigMode.DIRECT:
            # 直接配置模式（可选）
            if namespace.pg_url:
                if not NamespaceService.validate_pg_url(namespace.pg_url):
                    raise ValueError("无效的PostgreSQL URL格式")
                pg_config['url'] = namespace.pg_url
                logger.debug(f"PG URL: {NamespaceService.mask_url_password(namespace.pg_url)}")
            if namespace.pg_nacos_key:
                raise ValueError("PostgreSQL直接配置模式下不应提供pg_nacos_key")

        elif pg_config_mode == ConfigMode.NACOS:
            # Nacos配置模式（可选）
            if namespace.pg_nacos_key:
                pg_config['nacos_key'] = namespace.pg_nacos_key
                logger.info(f"PostgreSQL Nacos模式：存储配置键 '{namespace.pg_nacos_key}'")
            if namespace.pg_url:
                raise ValueError("PostgreSQL Nacos配置模式下不应提供pg_url")

        # 创建命名空间
        ns = Namespace(
            name=namespace.name,
            description=namespace.description,
            redis_config=redis_config,
            pg_config=pg_config,
            version=1
        )
        session.add(ns)
        await session.commit()
        await session.refresh(ns)

        logger.info(f"成功创建命名空间: {namespace.name}")
        return NamespaceService._build_namespace_response(ns)

    @staticmethod
    async def get_namespace(
        session: AsyncSession,
        namespace_name: str
    ) -> NamespaceResponse:
        """
        获取指定命名空间的详细信息

        Args:
            session: 数据库会话
            namespace_name: 命名空间名称

        Returns:
            命名空间信息

        Raises:
            ValueError: 命名空间不存在
        """
        logger.debug(f"获取命名空间详情: {namespace_name}")

        stmt = select(Namespace).where(Namespace.name == namespace_name)
        result = await session.execute(stmt)
        ns = result.scalar_one_or_none()

        if not ns:
            logger.error(f"命名空间 '{namespace_name}' 不存在")
            raise ValueError(f"命名空间 '{namespace_name}' 不存在")

        return NamespaceService._build_namespace_response(ns)

    @staticmethod
    async def update_namespace(
        session: AsyncSession,
        namespace_name: str,
        namespace: NamespaceUpdate
    ) -> NamespaceResponse:
        """
        更新命名空间配置

        Args:
            session: 数据库会话
            namespace_name: 命名空间名称
            namespace: 更新的配置信息

        Returns:
            更新后的命名空间信息

        Raises:
            ValueError: 命名空间不存在、配置无效等
        """
        logger.info(f"更新命名空间: {namespace_name}")

        # 获取当前命名空间
        stmt = select(Namespace).where(Namespace.name == namespace_name)
        result = await session.execute(stmt)
        ns = result.scalar_one_or_none()

        if not ns:
            logger.error(f"命名空间 '{namespace_name}' 不存在")
            raise ValueError(f"命名空间 '{namespace_name}' 不存在")

        # 获取当前配置
        current_redis_config = ns.redis_config if ns.redis_config else {}
        current_pg_config = ns.pg_config if ns.pg_config else {}

        # 确定 Redis 和 PostgreSQL 的配置模式
        redis_mode = None
        pg_mode = None

        if namespace.redis_config_mode is not None:
            redis_mode = namespace.redis_config_mode
        elif namespace.config_mode is not None:
            redis_mode = namespace.config_mode

        if namespace.pg_config_mode is not None:
            pg_mode = namespace.pg_config_mode
        elif namespace.config_mode is not None:
            pg_mode = namespace.config_mode

        has_updates = False

        # 更新描述
        if namespace.description is not None:
            ns.description = namespace.description
            has_updates = True

        # 处理 Redis 配置
        if redis_mode is not None:
            # 更新 Redis 配置模式
            redis_config = {'config_mode': redis_mode.value}

            if redis_mode == ConfigMode.DIRECT:
                # 直接配置模式
                if namespace.redis_nacos_key:
                    raise ValueError("Redis直接配置模式下不应提供redis_nacos_key")

                if namespace.redis_url:
                    if not NamespaceService.validate_redis_url(namespace.redis_url):
                        raise ValueError("无效的Redis URL格式")
                    redis_config['url'] = namespace.redis_url
                    logger.debug(f"更新 Redis URL: {NamespaceService.mask_url_password(namespace.redis_url)}")
                else:
                    # 保留原有URL
                    redis_config['url'] = current_redis_config.get('url', '')

            elif redis_mode == ConfigMode.NACOS:
                # Nacos配置模式
                if namespace.redis_url:
                    raise ValueError("Redis Nacos配置模式下不应提供redis_url")

                if namespace.redis_nacos_key:
                    redis_config['nacos_key'] = namespace.redis_nacos_key
                    logger.info(f"Nacos模式：更新Redis配置键为 '{namespace.redis_nacos_key}'")
                else:
                    # 保留原有配置
                    if current_redis_config.get('nacos_key'):
                        redis_config['nacos_key'] = current_redis_config.get('nacos_key')

            ns.redis_config = redis_config
            has_updates = True
        else:
            # 没有提供Redis配置模式，保持当前模式并验证更新字段
            current_redis_mode = current_redis_config.get('config_mode', 'direct')

            if current_redis_mode == 'direct':
                if namespace.redis_nacos_key:
                    raise ValueError("Redis当前为直接配置模式，不能提供redis_nacos_key")

                if namespace.redis_url:
                    if not NamespaceService.validate_redis_url(namespace.redis_url):
                        raise ValueError("无效的Redis URL格式")
                    current_redis_config['url'] = namespace.redis_url
                    ns.redis_config = current_redis_config
                    has_updates = True
                    logger.debug(f"更新 Redis URL: {NamespaceService.mask_url_password(namespace.redis_url)}")

            else:  # nacos mode
                if namespace.redis_url:
                    raise ValueError("Redis当前为Nacos配置模式，不能提供redis_url")

                if namespace.redis_nacos_key:
                    current_redis_config['nacos_key'] = namespace.redis_nacos_key
                    ns.redis_config = current_redis_config
                    has_updates = True
                    logger.info(f"Nacos模式：更新Redis配置键为 '{namespace.redis_nacos_key}'")

        # 处理 PostgreSQL 配置
        if pg_mode is not None:
            # 更新 PostgreSQL 配置模式
            pg_config = {'config_mode': pg_mode.value}

            if pg_mode == ConfigMode.DIRECT:
                # 直接配置模式
                if namespace.pg_nacos_key:
                    raise ValueError("PostgreSQL直接配置模式下不应提供pg_nacos_key")

                if namespace.pg_url:
                    if not NamespaceService.validate_pg_url(namespace.pg_url):
                        raise ValueError("无效的PostgreSQL URL格式")
                    pg_config['url'] = namespace.pg_url
                    logger.debug(f"更新 PG URL: {NamespaceService.mask_url_password(namespace.pg_url)}")
                elif current_pg_config.get('url'):
                    pg_config['url'] = current_pg_config.get('url')

            elif pg_mode == ConfigMode.NACOS:
                # Nacos配置模式
                if namespace.pg_url:
                    raise ValueError("PostgreSQL Nacos配置模式下不应提供pg_url")

                if namespace.pg_nacos_key:
                    pg_config['nacos_key'] = namespace.pg_nacos_key
                    logger.info(f"Nacos模式：更新PG配置键为 '{namespace.pg_nacos_key}'")
                else:
                    if current_pg_config.get('nacos_key'):
                        pg_config['nacos_key'] = current_pg_config.get('nacos_key')

            ns.pg_config = pg_config
            has_updates = True
        else:
            # 没有提供PostgreSQL配置模式，保持当前模式并验证更新字段
            current_pg_mode = current_pg_config.get('config_mode', 'direct')

            if current_pg_mode == 'direct':
                if namespace.pg_nacos_key:
                    raise ValueError("PostgreSQL当前为直接配置模式，不能提供pg_nacos_key")

                if namespace.pg_url:
                    if not NamespaceService.validate_pg_url(namespace.pg_url):
                        raise ValueError("无效的PostgreSQL URL格式")
                    current_pg_config['url'] = namespace.pg_url
                    ns.pg_config = current_pg_config
                    has_updates = True
                    logger.debug(f"更新 PG URL: {NamespaceService.mask_url_password(namespace.pg_url)}")

            else:  # nacos mode
                if namespace.pg_url:
                    raise ValueError("PostgreSQL当前为Nacos配置模式，不能提供pg_url")

                if namespace.pg_nacos_key:
                    current_pg_config['nacos_key'] = namespace.pg_nacos_key
                    ns.pg_config = current_pg_config
                    has_updates = True
                    logger.info(f"Nacos模式：更新PG配置键为 '{namespace.pg_nacos_key}'")

        # 更新启用状态
        if namespace.enabled is not None:
            ns.is_active = namespace.enabled
            has_updates = True

        if not has_updates:
            raise ValueError("没有提供要更新的字段")

        # 更新版本号
        ns.version = (ns.version or 0) + 1

        await session.commit()
        await session.refresh(ns)

        logger.info(f"成功更新命名空间: {namespace_name}")
        return NamespaceService._build_namespace_response(ns)

    @staticmethod
    async def delete_namespace(
        session: AsyncSession,
        namespace_name: str
    ) -> Dict[str, str]:
        """
        删除命名空间

        Args:
            session: 数据库会话
            namespace_name: 命名空间名称

        Returns:
            删除结果

        Raises:
            ValueError: 命名空间不存在或不能删除默认命名空间
        """
        if namespace_name == 'default':
            logger.error("尝试删除默认命名空间")
            raise ValueError("不能删除默认命名空间")

        logger.info(f"删除命名空间: {namespace_name}")

        # 检查命名空间是否存在
        stmt = select(Namespace).where(Namespace.name == namespace_name)
        result = await session.execute(stmt)
        ns = result.scalar_one_or_none()

        if not ns:
            logger.error(f"命名空间 '{namespace_name}' 不存在")
            raise ValueError(f"命名空间 '{namespace_name}' 不存在")

        # 删除命名空间
        await session.delete(ns)
        await session.commit()

        logger.info(f"成功删除命名空间: {namespace_name}")
        return {"message": f"命名空间 '{namespace_name}' 已删除"}

    @staticmethod
    async def activate_namespace(
        session: AsyncSession,
        namespace_name: str
    ) -> Dict[str, str]:
        """
        激活命名空间

        Args:
            session: 数据库会话
            namespace_name: 命名空间名称

        Returns:
            激活结果

        Raises:
            ValueError: 命名空间不存在
        """
        logger.info(f"激活命名空间: {namespace_name}")

        stmt = (
            update(Namespace)
            .where(Namespace.name == namespace_name)
            .values(is_active=True)
        )
        result = await session.execute(stmt)

        if result.rowcount == 0:
            logger.error(f"命名空间 '{namespace_name}' 不存在")
            raise ValueError(f"命名空间 '{namespace_name}' 不存在")

        await session.commit()

        logger.info(f"成功激活命名空间: {namespace_name}")
        return {"message": f"命名空间 '{namespace_name}' 已激活"}

    @staticmethod
    async def deactivate_namespace(
        session: AsyncSession,
        namespace_name: str
    ) -> Dict[str, str]:
        """
        停用命名空间

        Args:
            session: 数据库会话
            namespace_name: 命名空间名称

        Returns:
            停用结果

        Raises:
            ValueError: 命名空间不存在或不能停用默认命名空间
        """
        if namespace_name == 'default':
            logger.error("尝试停用默认命名空间")
            raise ValueError("不能停用默认命名空间")

        logger.info(f"停用命名空间: {namespace_name}")

        stmt = (
            update(Namespace)
            .where(Namespace.name == namespace_name)
            .values(is_active=False)
        )
        result = await session.execute(stmt)

        if result.rowcount == 0:
            logger.error(f"命名空间 '{namespace_name}' 不存在")
            raise ValueError(f"命名空间 '{namespace_name}' 不存在")

        await session.commit()

        logger.info(f"成功停用命名空间: {namespace_name}")
        return {"message": f"命名空间 '{namespace_name}' 已停用"}

    @staticmethod
    async def get_namespace_statistics(
        session: AsyncSession,  # noqa: ARG004 - 预留给未来实现
        namespace_name: str
    ) -> Dict[str, Any]:
        """
        获取命名空间统计信息

        Args:
            session: 数据库会话（预留给未来实现）
            namespace_name: 命名空间名称

        Returns:
            统计信息（符合 NamespaceStatisticsResponse 格式）

        注意:
            这是一个占位实现，返回模拟数据。
            实际使用时需要连接到命名空间的 Redis/PostgreSQL 查询真实统计数据。
        """
        _ = session  # 预留给未来实现，避免 linter 警告
        logger.debug(f"获取命名空间统计信息: {namespace_name}")

        # TODO: 实现真实的统计逻辑
        # 1. 获取命名空间配置
        # 2. 连接到 Redis 和 PostgreSQL
        # 3. 查询队列数、任务数、Worker 数等统计信息

        # 暂时返回模拟数据
        return {
            "success": True,
            "data": {
                "total_queues": 0,
                "total_tasks": 0,
                "active_workers": 0,
                "pending_tasks": 0,
                "processing_tasks": 0,
                "completed_tasks": 0,
                "failed_tasks": 0
            },
            "namespace": namespace_name,
            "timestamp": datetime.now().isoformat()
        }

    @staticmethod
    async def batch_activate_namespaces(
        session: AsyncSession,
        namespace_names: List[str]
    ) -> Dict[str, Any]:
        """
        批量激活命名空间

        Args:
            session: 数据库会话
            namespace_names: 命名空间名称列表

        Returns:
            激活结果，包含成功激活的数量和命名空间列表
        """
        logger.info(f"批量激活命名空间: {namespace_names}")

        stmt = (
            update(Namespace)
            .where(Namespace.name.in_(namespace_names))
            .values(is_active=True)
        )
        result = await session.execute(stmt)
        await session.commit()

        activated_count = result.rowcount
        logger.info(f"批量激活了 {activated_count} 个命名空间")

        return {
            "activated": activated_count,
            "namespaces": namespace_names[:activated_count]
        }

    @staticmethod
    async def batch_deactivate_namespaces(
        session: AsyncSession,
        namespace_names: List[str]
    ) -> Dict[str, Any]:
        """
        批量停用命名空间

        Args:
            session: 数据库会话
            namespace_names: 命名空间名称列表

        Returns:
            停用结果，包含成功停用的数量、命名空间列表和被跳过的列表
        """
        logger.info(f"批量停用命名空间: {namespace_names}")

        # 过滤掉默认命名空间
        filtered_names = [name for name in namespace_names if name != 'default']

        if not filtered_names:
            logger.warning("批量停用操作中所有命名空间都被跳过（包含 default）")
            return {
                "deactivated": 0,
                "namespaces": [],
                "skipped": ["default"]
            }

        stmt = (
            update(Namespace)
            .where(Namespace.name.in_(filtered_names))
            .values(is_active=False)
        )
        result = await session.execute(stmt)
        await session.commit()

        deactivated_count = result.rowcount
        logger.info(f"批量停用了 {deactivated_count} 个命名空间")

        response = {
            "deactivated": deactivated_count,
            "namespaces": filtered_names[:deactivated_count]
        }

        # 如果有被跳过的
        skipped = [name for name in namespace_names if name not in filtered_names]
        if skipped:
            response["skipped"] = skipped

        return response


__all__ = ['NamespaceService']



"""
服务基类
提供公共的时间范围处理方法
"""
from datetime import datetime, timedelta, timezone
from typing import Optional, List, Dict, Any
import logging

from jettask.schemas import TimeRangeQuery

logger = logging.getLogger(__name__)


class TimeRangeResult:
    """时间范围处理结果"""
    def __init__(self, start_time: datetime, end_time: datetime, interval: str, interval_seconds: int, granularity: str):
        self.start_time = start_time
        self.end_time = end_time
        self.interval = interval
        self.interval_seconds = interval_seconds
        self.granularity = granularity


class QueueResolutionResult:
    """队列解析结果"""
    def __init__(
        self,
        target_queues: List[str],
        all_priority_queues: List[str],
        base_queue_map: Dict[str, str]
    ):
        """
        Args:
            target_queues: 有效的基础队列列表
            all_priority_queues: 展开后的所有优先级队列列表
            base_queue_map: 优先级队列到基础队列的映射
        """
        self.target_queues = target_queues
        self.all_priority_queues = all_priority_queues
        self.base_queue_map = base_queue_map


class BaseService:
    """服务基类，提供公共的集成方法"""

    @staticmethod
    def _parse_time_range(time_range: str, end_time: datetime) -> datetime:
        """
        解析时间范围字符串

        Args:
            time_range: 时间范围字符串，如 "24h", "7d", "30m"
            end_time: 结束时间

        Returns:
            开始时间
        """
        if time_range.endswith('m'):
            minutes = int(time_range[:-1])
            return end_time - timedelta(minutes=minutes)
        elif time_range.endswith('h'):
            hours = int(time_range[:-1])
            return end_time - timedelta(hours=hours)
        elif time_range.endswith('d'):
            days = int(time_range[:-1])
            return end_time - timedelta(days=days)
        else:
            return end_time - timedelta(hours=24)  # 默认24小时

    @staticmethod
    def _parse_time_range_query(query: TimeRangeQuery) -> TimeRangeResult:
        """
        解析TimeRangeQuery对象

        Args:
            query: 时间范围查询对象

        Returns:
            时间范围处理结果
        """
        # 如果有结束时间，使用它；否则使用当前时间
        if query.end_time:
            end_time = datetime.fromisoformat(query.end_time.replace('Z', '+00:00')) if isinstance(query.end_time, str) else query.end_time
        else:
            end_time = datetime.now(timezone.utc)

        # 如果有开始时间，使用它；否则基于interval或默认24小时
        if query.start_time:
            start_time = datetime.fromisoformat(query.start_time.replace('Z', '+00:00')) if isinstance(query.start_time, str) else query.start_time
        else:
            # 使用interval字段来计算开始时间，默认为24小时
            interval = query.interval or "24h"
            start_time = BaseService._parse_time_range(interval, end_time)

        return BaseService._calculate_dynamic_interval(start_time, end_time)

    @staticmethod
    def _calculate_dynamic_interval(start_time: datetime, end_time: datetime, target_points: int = 200) -> TimeRangeResult:
        """
        根据时间范围动态计算合适的时间间隔

        Args:
            start_time: 开始时间
            end_time: 结束时间
            target_points: 目标数据点数量，默认200

        Returns:
            时间范围处理结果，包含计算出的最佳间隔
        """
        duration = (end_time - start_time).total_seconds()
        ideal_interval_seconds = duration / target_points

        # 选择合适的间隔
        intervals = [
            (1, '1 seconds', 'second'),
            (5, '5 seconds', 'second'),
            (10, '10 seconds', 'second'),
            (30, '30 seconds', 'second'),
            (60, '1 minute', 'minute'),
            (300, '5 minutes', 'minute'),
            (600, '10 minutes', 'minute'),
            (1800, '30 minutes', 'minute'),
            (3600, '1 hour', 'hour'),
            (21600, '6 hours', 'hour'),
            (43200, '12 hours', 'hour'),
            (86400, '1 day', 'day')
        ]

        for seconds, interval_str, granularity in intervals:
            if ideal_interval_seconds <= seconds:
                return TimeRangeResult(start_time, end_time, interval_str, seconds, granularity)

        # 默认返回1天间隔
        return TimeRangeResult(start_time, end_time, '1 day', 86400, 'day')

    @classmethod
    def _resolve_time_range(
        cls,
        start_time: Optional[datetime] = None,
        end_time: Optional[datetime] = None,
        time_range: Optional[str] = None,
        default_range: str = "1h",
        target_points: int = 200,
        min_interval_seconds: int = 0
    ) -> TimeRangeResult:
        """
        统一处理时间范围参数的高级方法

        支持三种输入方式：
        1. 仅传入 time_range 字符串（如 "15m", "1h", "24h", "7d"）
        2. 仅传入 start_time 和 end_time
        3. 都不传入，使用默认时间范围

        Args:
            start_time: 开始时间（可选）
            end_time: 结束时间（可选）
            time_range: 时间范围字符串（可选），如 "15m", "1h", "24h", "7d"
            default_range: 默认时间范围字符串，默认 "1h"
            target_points: 目标数据点数量，默认200
            min_interval_seconds: 最小间隔秒数，默认0（无限制）。
                                  设置为60表示最小粒度为分钟。

        Returns:
            TimeRangeResult: 包含解析后的时间范围和计算出的最佳间隔

        Examples:
            # 使用时间范围字符串
            result = cls._resolve_time_range(time_range="24h")

            # 使用具体时间
            result = cls._resolve_time_range(start_time=start, end_time=end)

            # 使用默认时间范围
            result = cls._resolve_time_range()

            # 自定义默认范围和目标点数
            result = cls._resolve_time_range(default_range="15m", target_points=100)

            # 设置最小粒度为分钟（60秒）
            result = cls._resolve_time_range(time_range="1h", min_interval_seconds=60)
        """
        now = datetime.now(timezone.utc)

        # 处理时间范围
        if time_range and not start_time:
            # 优先使用 time_range 字符串
            start_time = cls._parse_time_range(time_range, now)
            end_time = now
        elif not start_time:
            # 使用默认时间范围
            start_time = cls._parse_time_range(default_range, now)
            end_time = now

        # 确保 end_time 有值
        if not end_time:
            end_time = now

        # 计算动态时间间隔
        result = cls._calculate_dynamic_interval(start_time, end_time, target_points)

        # 如果设置了最小间隔，确保不低于最小值
        if min_interval_seconds > 0 and result.interval_seconds < min_interval_seconds:
            # 重新计算，使用最小间隔
            granularity = "second"
            if min_interval_seconds >= 86400:
                granularity = "day"
            elif min_interval_seconds >= 3600:
                granularity = "hour"
            elif min_interval_seconds >= 60:
                granularity = "minute"

            # 构建间隔描述
            if min_interval_seconds == 60:
                interval_str = "1 minute"
            elif min_interval_seconds == 300:
                interval_str = "5 minutes"
            elif min_interval_seconds == 600:
                interval_str = "10 minutes"
            elif min_interval_seconds == 1800:
                interval_str = "30 minutes"
            elif min_interval_seconds == 3600:
                interval_str = "1 hour"
            else:
                interval_str = f"{min_interval_seconds} seconds"

            logger.debug(
                f"时间间隔 {result.interval_seconds}s 小于最小值 {min_interval_seconds}s，"
                f"已调整为 {interval_str}"
            )

            result = TimeRangeResult(
                start_time=result.start_time,
                end_time=result.end_time,
                interval=interval_str,
                interval_seconds=min_interval_seconds,
                granularity=granularity
            )

        return result

    @staticmethod
    async def _resolve_queues(
        queues: List[str],
        registry,
        expand_priority: bool = True
    ) -> QueueResolutionResult:
        """
        统一处理队列名称解析

        验证队列是否存在于注册表中，并可选择性地展开为优先级队列。

        Args:
            queues: 请求的基础队列名称列表
            registry: QueueRegistry 实例
            expand_priority: 是否展开为优先级队列，默认 True

        Returns:
            QueueResolutionResult: 包含解析后的队列信息

        Examples:
            # 展开优先级队列
            result = await cls._resolve_queues(["email_queue"], registry)
            # result.target_queues = ["email_queue"]
            # result.all_priority_queues = ["email_queue:1", "email_queue:2", "email_queue:3"]
            # result.base_queue_map = {"email_queue:1": "email_queue", ...}

            # 不展开优先级队列
            result = await cls._resolve_queues(["email_queue"], registry, expand_priority=False)
            # result.target_queues = ["email_queue"]
            # result.all_priority_queues = ["email_queue"]
            # result.base_queue_map = {"email_queue": "email_queue"}
        """
        # 获取注册表中的所有基础队列
        all_base_queues = await registry.get_base_queues()

        # 过滤出存在于注册表中的队列
        target_queues = [q for q in queues if q in all_base_queues]

        if not expand_priority:
            # 不展开优先级队列
            base_queue_map = {q: q for q in target_queues}
            return QueueResolutionResult(
                target_queues=target_queues,
                all_priority_queues=target_queues,
                base_queue_map=base_queue_map
            )

        # 展开基础队列到所有优先级队列
        all_priority_queues = []
        base_queue_map = {}  # priority_queue -> base_queue 的映射

        for base_queue in target_queues:
            priority_queues = await registry.get_priority_queues_for_base(base_queue)
            if priority_queues:
                all_priority_queues.extend(priority_queues)
                for pq in priority_queues:
                    base_queue_map[pq] = base_queue
            else:
                # 如果没有优先级队列，可能队列名本身就是完整的
                all_priority_queues.append(base_queue)
                base_queue_map[base_queue] = base_queue

        logger.debug(f"队列解析: {queues} -> target={target_queues}, priority={all_priority_queues}")

        return QueueResolutionResult(
            target_queues=target_queues,
            all_priority_queues=all_priority_queues,
            base_queue_map=base_queue_map
        )



"""
Namespace 模型

对应 namespaces 表，用于存储命名空间配置
"""
from sqlalchemy import Column, String, Integer, Text, Boolean, TIMESTAMP, func
from sqlalchemy.dialects.postgresql import JSONB
from datetime import datetime, timezone
from typing import Optional, Dict, Any

from ..base import Base


class Namespace(Base):
    """
    命名空间配置表

    存储各个命名空间的配置信息，包括 Redis 和 PostgreSQL 连接配置
    """
    __tablename__ = 'namespaces'

    # 主键
    id = Column(Integer, primary_key=True, autoincrement=True, comment='自增主键')

    # 命名空间名称（唯一）
    name = Column(Text, unique=True, nullable=False, comment='命名空间名称')

    # 描述
    description = Column(Text, nullable=True, comment='命名空间描述')

    # Redis 配置（JSONB 格式，包含 config_mode, url, nacos_key 等）
    redis_config = Column(JSONB, nullable=True, comment='Redis 配置')

    # PostgreSQL 配置（JSONB 格式，包含 config_mode, url, nacos_key 等）
    pg_config = Column(JSONB, nullable=True, comment='PostgreSQL 配置')

    # 是否激活
    is_active = Column(Boolean, nullable=False, default=True, comment='是否激活')

    # 配置版本号
    version = Column(Integer, nullable=False, default=1, comment='配置版本号')

    # 时间戳
    created_at = Column(
        TIMESTAMP(timezone=True),
        nullable=False,
        server_default=func.current_timestamp(),
        comment='创建时间'
    )
    updated_at = Column(
        TIMESTAMP(timezone=True),
        nullable=False,
        server_default=func.current_timestamp(),
        onupdate=func.current_timestamp(),
        comment='更新时间'
    )

    def get_redis_config_mode(self) -> str:
        """获取 Redis 配置模式"""
        if self.redis_config:
            return self.redis_config.get('config_mode', 'direct')
        return 'direct'

    def get_redis_url(self) -> Optional[str]:
        """获取 Redis URL（仅 direct 模式）"""
        if self.redis_config and self.get_redis_config_mode() == 'direct':
            return self.redis_config.get('url')
        return None

    def get_redis_nacos_key(self) -> Optional[str]:
        """获取 Redis Nacos Key（仅 nacos 模式）"""
        if self.redis_config and self.get_redis_config_mode() == 'nacos':
            return self.redis_config.get('nacos_key')
        return None

    def get_pg_config_mode(self) -> str:
        """获取 PostgreSQL 配置模式"""
        if self.pg_config:
            return self.pg_config.get('config_mode', 'direct')
        return 'direct'

    def get_pg_url(self) -> Optional[str]:
        """获取 PostgreSQL URL（仅 direct 模式）"""
        if self.pg_config and self.get_pg_config_mode() == 'direct':
            return self.pg_config.get('url')
        return None

    def get_pg_nacos_key(self) -> Optional[str]:
        """获取 PostgreSQL Nacos Key（仅 nacos 模式）"""
        if self.pg_config and self.get_pg_config_mode() == 'nacos':
            return self.pg_config.get('nacos_key')
        return None

    def to_dict(self) -> Dict[str, Any]:
        """转换为字典"""
        return {
            'id': self.id,
            'name': self.name,
            'description': self.description,
            'redis_config': self.redis_config,
            'pg_config': self.pg_config,
            'is_active': self.is_active,
            'version': self.version,
            'created_at': self.created_at.isoformat() if self.created_at else None,
            'updated_at': self.updated_at.isoformat() if self.updated_at else None,
        }

    def __repr__(self) -> str:
        return f"<Namespace(id={self.id}, name='{self.name}', is_active={self.is_active})>"



"""
API v1 路由模块集合

路由结构：
- /api/v1/namespaces - 命名空间列表和创建（全局）
- /api/v1/{namespace} - 命名空间详情、更新、删除（命名空间下）
- /api/v1/{namespace}/statistics - 命名空间统计（命名空间下）
- /api/v1/{namespace}/queues - 队列管理（命名空间下的资源）
- /api/v1/{namespace}/queues/send - 任务发送（命名空间下的资源，已合并到 queues）
- /api/v1/{namespace}/sql-history - SQL 历史查询（命名空间下的资源）
- /api/v1/{namespace}/scheduled - 定时任务（命名空间下的资源）
- /api/v1/{namespace}/workers - Worker 监控（命名空间下的资源）
- /api/v1/{namespace}/settings - 设置（命名空间下的资源）
- /api/v1/{namespace}/alerts - 告警规则（命名空间下的资源）
- /api/v1/{namespace}/webhooks - Webhook 回调接收（命名空间下的资源）
- /api/v1/{namespace}/assets - 资产管理（命名空间下的资源）
"""
from fastapi import APIRouter

# 导入主要模块的路由
from .overview import router as overview_router                           # 概览
from .namespaces import global_router as namespaces_global_router         # 命名空间（全局路由）
from .namespaces import namespace_router as namespaces_namespace_router   # 命名空间（命名空间路由）
from .queues import router as queues_router                               # 队列（包含任务发送）
from .scheduled import router as scheduled_router                         # 定时任务
from .alerts import router as alerts_router                               # 告警
from .settings import router as settings_router                           # 设置
from .workers import router as workers_router                             # Worker 监控
from .sql_history import router as sql_history_router                     # SQL 历史查询
from .auth import router as auth_router                                   # 认证（登录、刷新token）
from .webhooks import router as webhooks_router                           # Webhook 回调接收
from .assets import router as assets_router                               # 资产管理

# 创建 v1 总路由，添加统一的 /api/v1 前缀
api_router = APIRouter(prefix="/api/task/v1")

# 1. 注册全局路由（不需要 namespace 的路由）
api_router.include_router(auth_router)               # 认证接口（登录、刷新token）
api_router.include_router(namespaces_global_router)  # 命名空间管理（列表和创建）
api_router.include_router(alerts_router)             # 告警规则管理（支持可选 namespace）

# 2. 创建命名空间路由（所有需要 namespace 的资源）
namespace_router = APIRouter(prefix="/{namespace}")

# 3. 注册命名空间下的资源路由
namespace_router.include_router(namespaces_namespace_router)  # 命名空间详情、更新、删除、统计
namespace_router.include_router(overview_router)              # 系统概览、健康检查
namespace_router.include_router(queues_router)                # 队列管理、任务处理、任务发送
namespace_router.include_router(sql_history_router)           # SQL 历史查询
namespace_router.include_router(scheduled_router)             # 定时任务管理
namespace_router.include_router(workers_router)               # Worker 监控
namespace_router.include_router(settings_router)              # 系统配置
namespace_router.include_router(webhooks_router)              # Webhook 回调接收
namespace_router.include_router(assets_router)                # 资产管理

# 4. 将命名空间路由注册到总路由
api_router.include_router(namespace_router)

__all__ = ['api_router']





"""
TaskRun 模型

对应 task_runs 表，用于存储任务执行记录
"""
from sqlalchemy import Column, String, Integer, Text, TIMESTAMP, Float, Index, ForeignKey
from sqlalchemy.dialects.postgresql import JSONB
from datetime import datetime, timezone
from typing import Optional, Dict, Any

from ..base import Base


class TaskRun(Base):
    """
    任务执行记录表

    存储每次任务执行的状态、结果和执行时间等信息
    """
    __tablename__ = 'task_runs'

    # 复合主键 - (task_name, trigger_time, stream_id)
    # 说明：
    # - task_runs 是分区表（按 trigger_time 分区），主键必须包含分区键
    # - 主键顺序按粒度从粗到细排列，以提高索引复用率：
    #   - task_name: 任务名称（粗粒度，可能只有几十种任务）
    #   - trigger_time: 任务触发时间（中粒度，分区键）
    #   - stream_id: Redis Stream 事件ID（细粒度，1:1对应任务）
    # 这样的顺序支持：
    #   - 按 task_name 查询（复用索引前缀）
    #   - 按 task_name + trigger_time 范围查询
    #   - 按 task_name + trigger_time + stream_id 精确查询
    task_name = Column(Text, primary_key=True, nullable=False, comment='任务名称（粗粒度）')
    trigger_time = Column(TIMESTAMP(timezone=True), primary_key=True, nullable=False, comment='任务触发时间（分区键）')
    stream_id = Column(Text, primary_key=True, comment='Redis Stream 事件ID（细粒度）')

    # 执行状态
    status = Column(String(50), nullable=True, comment='任务状态 (pending/running/success/failed/retrying)')

    # 执行结果
    result = Column(JSONB, nullable=True, comment='任务执行结果')
    error = Column(Text, nullable=True, comment='错误信息（如果失败）')

    # 执行时间
    started_at = Column(TIMESTAMP(timezone=True), nullable=True, comment='开始执行时间（TIMESTAMP 类型），可能因重试而变化')
    completed_at = Column(TIMESTAMP(timezone=True), nullable=True, comment='完成时间（TIMESTAMP 类型）')

    # 重试次数
    retries = Column(Integer, nullable=True, default=0, comment='重试次数')

    # 执行时长（秒）
    duration = Column(Float, nullable=True, comment='执行时长（秒）')

    # 消费者信息
    consumer = Column(Text, nullable=True, comment='执行该任务的消费者ID')

    # 记录创建和更新时间
    created_at = Column(
        TIMESTAMP(timezone=True),
        nullable=False,
        default=lambda: datetime.now(timezone.utc),
        comment='记录创建时间'
    )
    updated_at = Column(
        TIMESTAMP(timezone=True),
        nullable=False,
        default=lambda: datetime.now(timezone.utc),
        onupdate=lambda: datetime.now(timezone.utc),
        comment='记录更新时间'
    )

    # 索引
    # 注意：主键 (task_name, trigger_time, stream_id) 已自动创建索引
    # 可以复用主键索引支持：
    #   - 按 task_name 查询
    #   - 按 (task_name, trigger_time) 范围查询
    #   - 按 (task_name, trigger_time, stream_id) 精确查询
    __table_args__ = (
        Index('idx_task_runs_status', 'status'),
        Index('idx_task_runs_stream_id', 'stream_id'),  # 支持按 stream_id 反查
        Index('idx_task_runs_started_at', 'started_at'),
    )

    def to_dict(self) -> Dict[str, Any]:
        """转换为字典"""
        return {
            'task_name': self.task_name,
            'trigger_time': self.trigger_time.isoformat() if self.trigger_time else None,
            'stream_id': self.stream_id,
            'status': self.status,
            'result': self.result,
            'error': self.error,
            'started_at': self.started_at.isoformat() if self.started_at else None,
            'completed_at': self.completed_at.isoformat() if self.completed_at else None,
            'retries': self.retries,
            'duration': self.duration,
            'consumer': self.consumer,
            'created_at': self.created_at.isoformat() if self.created_at else None,
            'updated_at': self.updated_at.isoformat() if self.updated_at else None,
        }

    def __repr__(self) -> str:
        return f"<TaskRun(stream_id='{self.stream_id}', status='{self.status}')>"



"""
资产获取工具类

提供给任务使用的资产获取和负载均衡功能
通过 HTTP API 获取资产列表，无需直接连接数据库
"""
import asyncio
import random
import logging
from typing import Optional, Dict, Any, List, Callable, Awaitable
from datetime import datetime, timezone
from dataclasses import dataclass, field
import httpx

logger = logging.getLogger(__name__)


@dataclass
class NodeStatus:
    """节点状态"""
    asset_id: int
    name: str
    url: str
    config: Dict[str, Any]
    weight: int
    queue_remaining: int = 0
    is_available: bool = True
    last_check: Optional[datetime] = None
    error: Optional[str] = None


@dataclass
class AssetInfo:
    """资产信息"""
    id: int
    name: str
    asset_type: str
    asset_group: str
    config: Dict[str, Any]
    status: str
    weight: int


@dataclass
class AssetCache:
    """资产缓存"""
    assets: List[AssetInfo] = field(default_factory=list)
    last_refresh: Optional[datetime] = None


class AssetClient:
    """
    资产客户端

    通过 HTTP API 获取和管理资产，提供负载均衡功能

    Example:
        ```python
        from jettask.utils.asset import AssetClient

        @app.task(queue="image_gen")
        async def generate_image(prompt: str):
            # 创建资产客户端（使用 API Key 认证）
            client = AssetClient(
                api_base_url="http://localhost:8080",
                namespace="default",
                api_key="your-api-key"
            )

            # 获取最空闲的 ComfyUI 节点
            node = await client.acquire_compute_node(
                asset_group="comfyui",
                max_queue=0,      # 只选择空闲节点
                timeout=600       # 等待最多 10 分钟
            )

            if node is None:
                raise Exception("没有可用的 ComfyUI 节点")

            # 使用节点
            result = await call_comfyui(node.url, prompt)
            return result
        ```
    """

    def __init__(
        self,
        api_base_url: str,
        namespace: str,
        api_key: Optional[str] = None,
        jwt_token: Optional[str] = None,
        cache_ttl: int = 60,
        http_timeout: float = 10.0
    ):
        """
        初始化资产客户端

        Args:
            api_base_url: API 基础 URL（如 http://localhost:8080）
            namespace: 命名空间
            api_key: API Key（X-API-Key 认证方式）
            jwt_token: JWT Token（Bearer Token 认证方式）
            cache_ttl: 资产列表缓存时间（秒）
            http_timeout: HTTP 请求超时时间（秒）

        Note:
            api_key 和 jwt_token 二选一，优先使用 api_key
        """
        self.api_base_url = api_base_url.rstrip('/')
        self.namespace = namespace
        self.api_key = api_key
        self.jwt_token = jwt_token
        self.cache_ttl = cache_ttl
        self.http_timeout = http_timeout
        self._cache: Dict[str, AssetCache] = {}

    def _get_auth_headers(self) -> Dict[str, str]:
        """获取认证请求头"""
        headers = {}
        if self.api_key:
            headers['X-API-Key'] = self.api_key
        elif self.jwt_token:
            headers['Authorization'] = f'Bearer {self.jwt_token}'
        return headers

    def _get_api_url(self, path: str) -> str:
        """构建 API URL"""
        return f"{self.api_base_url}/api/task/v1/{self.namespace}{path}"

    async def get_assets(
        self,
        asset_group: str,
        asset_type: Optional[str] = None,
        status: str = "active",
        use_cache: bool = True
    ) -> List[AssetInfo]:
        """
        获取资产列表

        Args:
            asset_group: 资产分组
            asset_type: 资产类型（可选）
            status: 状态筛选
            use_cache: 是否使用缓存

        Returns:
            资产列表
        """
        cache_key = f"{asset_group}:{asset_type}:{status}"

        # 检查缓存
        if use_cache and cache_key in self._cache:
            cache = self._cache[cache_key]
            if cache.last_refresh:
                age = (datetime.now(timezone.utc) - cache.last_refresh).total_seconds()
                if age < self.cache_ttl:
                    return cache.assets

        # 通过 API 获取
        url = self._get_api_url(f"/assets/group/{asset_group}")
        params = {"status": status}
        headers = self._get_auth_headers()

        try:
            async with httpx.AsyncClient(timeout=self.http_timeout) as http_client:
                resp = await http_client.get(url, params=params, headers=headers)
                resp.raise_for_status()
                data = resp.json()

                assets = []
                for item in data.get("data", []):
                    # 按类型筛选
                    if asset_type and item.get("asset_type") != asset_type:
                        continue

                    assets.append(AssetInfo(
                        id=item["id"],
                        name=item["name"],
                        asset_type=item["asset_type"],
                        asset_group=item["asset_group"],
                        config=item.get("config", {}),
                        status=item["status"],
                        weight=item.get("weight", 1)
                    ))

                # 更新缓存
                self._cache[cache_key] = AssetCache(
                    assets=assets,
                    last_refresh=datetime.now(timezone.utc)
                )

                return assets

        except Exception as e:
            logger.error(f"获取资产列表失败: {e}")
            # 如果有缓存，返回缓存数据
            if cache_key in self._cache:
                return self._cache[cache_key].assets
            return []

    async def acquire_compute_node(
        self,
        asset_group: str,
        check_func: Optional[Callable[[str, Dict[str, Any]], Awaitable[int]]] = None,
        max_queue: int = 0,
        timeout: float = 300.0,
        poll_interval: float = 5.0,
        strategy: str = "least_busy"
    ) -> Optional[NodeStatus]:
        """
        获取计算节点（带负载均衡）

        会检查所有节点的负载情况，选择最空闲的节点。
        如果所有节点都忙，会等待直到有空闲节点或超时。

        Args:
            asset_group: 资产分组（如 "comfyui"）
            check_func: 检查节点队列的异步函数，签名: async (url, config) -> queue_remaining
                        如果不提供，使用默认的 ComfyUI 检查函数
            max_queue: 最大允许的队列数，为 0 表示只选择空闲节点
            timeout: 等待超时时间（秒）
            poll_interval: 轮询间隔（秒）
            strategy: 选择策略
                - "least_busy": 选择队列最少的节点（默认）
                - "random": 从可用节点中随机选择
                - "weighted": 按权重加权随机选择

        Returns:
            NodeStatus: 选中的节点状态，超时返回 None

        Example:
            ```python
            # 使用默认的 ComfyUI 检查函数
            node = await client.acquire_compute_node("comfyui")

            # 使用自定义检查函数
            async def check_sd_webui(url, config):
                async with httpx.AsyncClient() as client:
                    resp = await client.get(f"{url}/sdapi/v1/progress")
                    data = resp.json()
                    return 1 if data.get("state", {}).get("job") else 0

            node = await client.acquire_compute_node(
                "sd_webui",
                check_func=check_sd_webui
            )
            ```
        """
        if check_func is None:
            check_func = self._default_comfyui_check

        start_time = asyncio.get_event_loop().time()

        while True:
            # 检查超时
            elapsed = asyncio.get_event_loop().time() - start_time
            if elapsed >= timeout:
                logger.warning(f"获取计算节点超时: asset_group={asset_group}")
                return None

            # 获取所有活跃的计算节点
            assets = await self.get_assets(
                asset_group=asset_group,
                asset_type="compute_node",
                status="active",
                use_cache=False  # 获取节点时不使用缓存，确保状态最新
            )

            if not assets:
                logger.warning(f"没有可用的计算节点: asset_group={asset_group}")
                await asyncio.sleep(poll_interval)
                continue

            # 检查每个节点的状态
            node_statuses: List[NodeStatus] = []

            for asset in assets:
                url = asset.config.get("url", "")
                if not url:
                    continue

                status = NodeStatus(
                    asset_id=asset.id,
                    name=asset.name,
                    url=url,
                    config=asset.config,
                    weight=asset.weight,
                    last_check=datetime.now(timezone.utc)
                )

                try:
                    queue_remaining = await check_func(url, asset.config)
                    status.queue_remaining = queue_remaining
                    status.is_available = queue_remaining <= max_queue
                except Exception as e:
                    status.is_available = False
                    status.error = str(e)
                    logger.warning(f"检查节点 {asset.name} 失败: {e}")

                node_statuses.append(status)

            # 筛选可用节点
            available_nodes = [n for n in node_statuses if n.is_available]

            if not available_nodes:
                logger.debug(
                    f"所有节点都忙，等待 {poll_interval} 秒后重试... "
                    f"(节点状态: {[(n.name, n.queue_remaining) for n in node_statuses]})"
                )
                await asyncio.sleep(poll_interval)
                continue

            # 根据策略选择节点
            selected = self._select_node(available_nodes, strategy)

            logger.info(
                f"选中计算节点: {selected.name} (queue={selected.queue_remaining}, "
                f"可用节点数={len(available_nodes)}/{len(node_statuses)})"
            )

            return selected

    def _select_node(self, nodes: List[NodeStatus], strategy: str) -> NodeStatus:
        """根据策略选择节点"""
        if strategy == "random":
            return random.choice(nodes)

        elif strategy == "weighted":
            # 按权重加权随机
            total_weight = sum(n.weight for n in nodes)
            if total_weight == 0:
                return random.choice(nodes)

            r = random.uniform(0, total_weight)
            cumulative = 0
            for node in nodes:
                cumulative += node.weight
                if r <= cumulative:
                    return node
            return nodes[-1]

        else:  # least_busy（默认）
            # 选择队列最少的，队列相同时按权重选择
            nodes.sort(key=lambda n: (n.queue_remaining, -n.weight))
            return nodes[0]

    async def get_api_key(
        self,
        asset_group: str,
        strategy: str = "random"
    ) -> Optional[Dict[str, Any]]:
        """
        获取 API 密钥

        Args:
            asset_group: 资产分组（如 "openai"）
            strategy: 选择策略
                - "random": 随机选择（默认）
                - "weighted": 按权重选择

        Returns:
            包含 asset_id, name, config 的字典，或 None
        """
        assets = await self.get_assets(
            asset_group=asset_group,
            asset_type="api_key",
            status="active"
        )

        if not assets:
            return None

        if strategy == "weighted":
            # 按权重加权随机
            total_weight = sum(a.weight for a in assets)
            if total_weight == 0:
                selected = random.choice(assets)
            else:
                r = random.uniform(0, total_weight)
                cumulative = 0
                selected = assets[-1]
                for asset in assets:
                    cumulative += asset.weight
                    if r <= cumulative:
                        selected = asset
                        break
        else:  # random
            selected = random.choice(assets)

        return {
            "asset_id": selected.id,
            "name": selected.name,
            "config": selected.config
        }

    async def get_config(
        self,
        asset_group: str,
        name: Optional[str] = None
    ) -> Optional[Dict[str, Any]]:
        """
        获取配置项

        Args:
            asset_group: 资产分组
            name: 配置名称（可选，不指定则返回第一个）

        Returns:
            配置字典，或 None
        """
        assets = await self.get_assets(
            asset_group=asset_group,
            asset_type="config",
            status="active"
        )

        if not assets:
            return None

        if name:
            for asset in assets:
                if asset.name == name:
                    return asset.config
            return None

        return assets[0].config

    @staticmethod
    async def _default_comfyui_check(url: str, config: Dict[str, Any]) -> int:
        """
        默认的 ComfyUI 队列检查函数

        调用 GET /api/queue/remaining 接口，返回 queue_remaining
        """
        timeout = config.get("timeout", 5.0)

        async with httpx.AsyncClient(timeout=timeout) as client:
            resp = await client.get(f"{url}/api/queue/remaining")
            resp.raise_for_status()
            data = resp.json()
            return data.get("queue_remaining", 0)

    def clear_cache(self):
        """清空缓存"""
        self._cache.clear()


# 便捷函数
async def acquire_comfyui_node(
    api_base_url: str,
    namespace: str,
    asset_group: str = "comfyui",
    max_queue: int = 0,
    timeout: float = 300.0,
    api_key: Optional[str] = None,
    jwt_token: Optional[str] = None
) -> Optional[NodeStatus]:
    """
    获取 ComfyUI 节点的便捷函数

    Args:
        api_base_url: API 基础 URL（如 http://localhost:8080）
        namespace: 命名空间
        asset_group: 资产分组，默认 "comfyui"
        max_queue: 最大允许的队列数
        timeout: 超时时间
        api_key: API Key 认证（可选）
        jwt_token: JWT Token 认证（可选）

    Returns:
        NodeStatus 或 None

    Example:
        ```python
        from jettask.utils.asset import acquire_comfyui_node

        @app.task(queue="image_gen")
        async def generate_image(prompt: str):
            node = await acquire_comfyui_node(
                api_base_url="http://localhost:8080",
                namespace="default",
                api_key="your-api-key",
                timeout=600
            )

            if node is None:
                raise Exception("没有可用的 ComfyUI 节点")

            # 使用节点
            result = await call_comfyui(node.url, prompt)
            return result
        ```
    """
    client = AssetClient(api_base_url, namespace, api_key=api_key, jwt_token=jwt_token)
    return await client.acquire_compute_node(
        asset_group=asset_group,
        max_queue=max_queue,
        timeout=timeout
    )


async def get_api_key(
    api_base_url: str,
    namespace: str,
    asset_group: str,
    api_key: Optional[str] = None,
    jwt_token: Optional[str] = None
) -> Optional[Dict[str, Any]]:
    """
    获取 API 密钥的便捷函数

    Args:
        api_base_url: API 基础 URL
        namespace: 命名空间
        asset_group: 资产分组（如 "openai"）
        api_key: API Key 认证（可选）
        jwt_token: JWT Token 认证（可选）

    Returns:
        包含 asset_id, name, config 的字典，或 None

    Example:
        ```python
        from jettask.utils.asset import get_api_key

        @app.task(queue="chat")
        async def chat(message: str):
            key_info = await get_api_key(
                api_base_url="http://localhost:8080",
                namespace="default",
                asset_group="openai",
                api_key="your-api-key"
            )

            if key_info is None:
                raise Exception("没有可用的 OpenAI Key")

            api_key = key_info["config"]["api_key"]
            # 使用 API Key
            ...
        ```
    """
    client = AssetClient(api_base_url, namespace, api_key=api_key, jwt_token=jwt_token)
    return await client.get_api_key(asset_group)



"""
TaskCenter API 客户端

这个类是一个纯粹的 API 客户端，用于与 TaskCenter API 进行通信。
所有方法都是异步的，支持实时查询。
"""
import logging
from typing import Optional, Dict, Any, List
import aiohttp

logger = logging.getLogger(__name__)


class TaskCenterClient:
    """
    TaskCenter API 客户端

    这是一个轻量级的 HTTP 客户端，专门用于与 TaskCenter API 进行通信。

    特点：
    - 纯异步接口，所有方法都返回协程
    - 无状态设计，每次调用都是实时查询
    - 自动管理 HTTP 会话
    - 完整的错误处理和日志记录

    使用示例：
        ```python
        # 初始化客户端
        client = TaskCenterClient(task_center_url="http://localhost:8001")

        # 获取命名空间列表
        namespaces = await client.get_namespace_list()

        # 获取特定命名空间
        namespace = await client.get_namespace("default")

        # 获取统计信息
        stats = await client.get_namespace_statistics("default")

        # 使用完毕后关闭
        await client.close()
        ```

    或使用上下文管理器：
        ```python
        async with TaskCenterClient("http://localhost:8001") as client:
            namespaces = await client.get_namespace_list()
        ```
    """

    def __init__(self, task_center_url: str, api_key: Optional[str] = None):
        """
        初始化 TaskCenter API 客户端

        Args:
            task_center_url: TaskCenter 的基础 URL，例如 "http://localhost:8001"
                           不需要包含 /api/task/v1 路径，客户端会自动添加
            api_key: API密钥，用于请求鉴权（可选）

        示例：
            client = TaskCenterClient("http://localhost:8001")
            client = TaskCenterClient("https://taskcenter.example.com", api_key="your-api-key")
        """
        # 移除末尾的斜杠
        self.task_center_url = task_center_url.rstrip('/')
        self._base_api_url = f"{self.task_center_url}/api/task/v1"
        self._session: Optional[aiohttp.ClientSession] = None
        self._api_key = api_key

        logger.info(f"TaskCenter 客户端初始化: {self.task_center_url}")
        if api_key:
            logger.info("API 密钥已配置，将使用鉴权模式")

    async def _get_session(self) -> aiohttp.ClientSession:
        """获取或创建 HTTP 会话"""
        if self._session is None or self._session.closed:
            self._session = aiohttp.ClientSession()
            logger.debug("创建新的 HTTP 会话")
        return self._session

    def _get_headers(self) -> Dict[str, str]:
        """获取请求头（包含 API Key）"""
        headers = {}
        if self._api_key:
            headers['X-API-Key'] = self._api_key
        return headers

    async def get_namespace_list(
        self,
        page: int = 1,
        page_size: int = 100,
        is_active: Optional[bool] = None
    ) -> List[Dict[str, Any]]:
        """
        获取命名空间列表

        Args:
            page: 页码，从 1 开始，默认为 1
            page_size: 每页数量，范围 1-100，默认为 100
            is_active: 是否只返回激活的命名空间，None 表示返回所有

        Returns:
            命名空间列表，每个元素是一个包含命名空间完整信息的字典

        Raises:
            aiohttp.ClientError: 网络请求失败
            ValueError: API 返回错误状态码

        示例：
            # 获取所有命名空间
            all_namespaces = await client.get_namespace_list()

            # 只获取激活的命名空间
            active_namespaces = await client.get_namespace_list(is_active=True)

            # 分页获取
            first_page = await client.get_namespace_list(page=1, page_size=20)
        """
        url = f"{self._base_api_url}/namespaces/"
        params = {
            'page': page,
            'page_size': page_size
        }
        if is_active is not None:
            params['is_active'] = is_active

        logger.debug(f"获取命名空间列表: page={page}, page_size={page_size}, is_active={is_active}")

        try:
            session = await self._get_session()
            headers = self._get_headers()
            async with session.get(url, params=params, headers=headers) as resp:
                if resp.status == 200:
                    data = await resp.json()
                    logger.debug(f"成功获取 {len(data)} 个命名空间")
                    return data
                else:
                    error_text = await resp.text()
                    logger.error(f"获取命名空间列表失败: HTTP {resp.status}, {error_text}")
                    raise ValueError(f"获取命名空间列表失败: HTTP {resp.status}")
        except aiohttp.ClientError as e:
            logger.error(f"网络请求失败: {e}")
            raise
        except Exception as e:
            logger.error(f"获取命名空间列表失败: {e}")
            raise

    async def get_namespace(self, namespace_name: str) -> Dict[str, Any]:
        """
        获取特定命名空间的详细信息

        Args:
            namespace_name: 命名空间名称

        Returns:
            命名空间的完整配置信息，包括：
            - name: 命名空间名称
            - description: 描述
            - enabled: 是否启用
            - redis_config: Redis 配置
            - pg_config: PostgreSQL 配置
            - redis_url: Redis 连接 URL
            - pg_url: PostgreSQL 连接 URL
            - version: 配置版本号
            - created_at: 创建时间
            - updated_at: 更新时间

        Raises:
            ValueError: 命名空间不存在或 API 返回错误
            aiohttp.ClientError: 网络请求失败

        示例：
            namespace = await client.get_namespace("default")
            print(f"命名空间: {namespace['name']}")
            print(f"Redis URL: {namespace['redis_url']}")
            print(f"PG URL: {namespace['pg_url']}")
        """
        url = f"{self._base_api_url}/{namespace_name}/"

        logger.debug(f"获取命名空间详情: {namespace_name}")

        try:
            session = await self._get_session()
            headers = self._get_headers()
            async with session.get(url, headers=headers) as resp:
                if resp.status == 200:
                    data = await resp.json()
                    logger.info(f"成功获取命名空间 '{namespace_name}' 的详细信息")
                    return data
                elif resp.status == 404:
                    logger.error(f"命名空间 '{namespace_name}' 不存在")
                    raise ValueError(f"命名空间 '{namespace_name}' 不存在")
                else:
                    error_text = await resp.text()
                    logger.error(f"获取命名空间失败: HTTP {resp.status}, {error_text}")
                    raise ValueError(f"获取命名空间失败: HTTP {resp.status}")
        except aiohttp.ClientError as e:
            logger.error(f"网络请求失败: {e}")
            raise
        except ValueError:
            raise
        except Exception as e:
            logger.error(f"获取命名空间失败: {e}")
            raise

    async def get_namespace_statistics(self, namespace_name: str) -> Dict[str, Any]:
        """
        获取命名空间的统计信息

        Args:
            namespace_name: 命名空间名称

        Returns:
            统计信息字典，包含：
            - total_queues: 队列总数
            - total_tasks: 任务总数
            - active_workers: 活跃 Worker 数
            - redis_memory_usage: Redis 内存使用（字节）
            - db_connections: 数据库连接数

        Raises:
            ValueError: 命名空间不存在或 API 返回错误
            aiohttp.ClientError: 网络请求失败

        示例：
            stats = await client.get_namespace_statistics("default")
            print(f"队列数: {stats['total_queues']}")
            print(f"任务数: {stats['total_tasks']}")
            print(f"Worker 数: {stats['active_workers']}")
        """
        url = f"{self._base_api_url}/{namespace_name}/statistics"

        logger.debug(f"获取命名空间统计信息: {namespace_name}")

        try:
            session = await self._get_session()
            headers = self._get_headers()
            async with session.get(url, headers=headers) as resp:
                if resp.status == 200:
                    data = await resp.json()
                    logger.info(f"成功获取命名空间 '{namespace_name}' 的统计信息")
                    return data
                elif resp.status == 404:
                    logger.error(f"命名空间 '{namespace_name}' 不存在")
                    raise ValueError(f"命名空间 '{namespace_name}' 不存在")
                else:
                    error_text = await resp.text()
                    logger.error(f"获取统计信息失败: HTTP {resp.status}, {error_text}")
                    raise ValueError(f"获取统计信息失败: HTTP {resp.status}")
        except aiohttp.ClientError as e:
            logger.error(f"网络请求失败: {e}")
            raise
        except ValueError:
            raise
        except Exception as e:
            logger.error(f"获取统计信息失败: {e}")
            raise

    async def create_namespace(self, namespace_data: Dict[str, Any]) -> Dict[str, Any]:
        """
        创建新的命名空间

        Args:
            namespace_data: 命名空间配置数据，包含：
                - name: 命名空间名称（必需）
                - description: 描述（可选）
                - config_mode: 配置模式，"direct" 或 "nacos"（必需）
                - redis_url: Redis 连接 URL（direct 模式必需）
                - pg_url: PostgreSQL 连接 URL（direct 模式必需）
                - redis_nacos_key: Redis Nacos 配置键（nacos 模式必需）
                - pg_nacos_key: PostgreSQL Nacos 配置键（nacos 模式必需）

        Returns:
            创建成功的命名空间信息

        Raises:
            ValueError: 创建失败（如名称已存在、参数错误等）
            aiohttp.ClientError: 网络请求失败

        示例：
            # 使用 direct 模式创建
            namespace = await client.create_namespace({
                "name": "production",
                "description": "生产环境",
                "config_mode": "direct",
                "redis_url": "redis://:password@localhost:6379/0",
                "pg_url": "postgresql://user:password@localhost:5432/jettask"
            })
        """
        url = f"{self._base_api_url}/namespaces/"

        logger.debug(f"创建命名空间: {namespace_data.get('name')}")

        try:
            session = await self._get_session()
            headers = self._get_headers()
            async with session.post(url, json=namespace_data, headers=headers) as resp:
                if resp.status == 201:
                    data = await resp.json()
                    logger.info(f"成功创建命名空间 '{namespace_data.get('name')}'")
                    return data
                elif resp.status == 400:
                    error_text = await resp.text()
                    logger.error(f"创建命名空间失败: {error_text}")
                    raise ValueError(f"创建命名空间失败: {error_text}")
                else:
                    error_text = await resp.text()
                    logger.error(f"创建命名空间失败: HTTP {resp.status}, {error_text}")
                    raise ValueError(f"创建命名空间失败: HTTP {resp.status}")
        except aiohttp.ClientError as e:
            logger.error(f"网络请求失败: {e}")
            raise
        except ValueError:
            raise
        except Exception as e:
            logger.error(f"创建命名空间失败: {e}")
            raise

    async def update_namespace(
        self,
        namespace_name: str,
        update_data: Dict[str, Any]
    ) -> Dict[str, Any]:
        """
        更新命名空间配置

        Args:
            namespace_name: 命名空间名称
            update_data: 要更新的字段，所有字段都是可选的：
                - description: 描述
                - enabled: 是否启用
                - config_mode: 配置模式
                - redis_url: Redis 连接 URL
                - pg_url: PostgreSQL 连接 URL
                - redis_nacos_key: Redis Nacos 配置键
                - pg_nacos_key: PostgreSQL Nacos 配置键

        Returns:
            更新后的命名空间信息

        Raises:
            ValueError: 更新失败（如命名空间不存在、参数错误等）
            aiohttp.ClientError: 网络请求失败

        示例：
            # 更新描述和状态
            namespace = await client.update_namespace("default", {
                "description": "默认命名空间（已更新）",
                "enabled": True
            })
        """
        url = f"{self._base_api_url}/{namespace_name}/"

        logger.debug(f"更新命名空间: {namespace_name}")

        try:
            session = await self._get_session()
            headers = self._get_headers()
            async with session.put(url, json=update_data, headers=headers) as resp:
                if resp.status == 200:
                    data = await resp.json()
                    logger.info(f"成功更新命名空间 '{namespace_name}'")
                    return data
                elif resp.status == 404:
                    logger.error(f"命名空间 '{namespace_name}' 不存在")
                    raise ValueError(f"命名空间 '{namespace_name}' 不存在")
                elif resp.status == 400:
                    error_text = await resp.text()
                    logger.error(f"更新命名空间失败: {error_text}")
                    raise ValueError(f"更新命名空间失败: {error_text}")
                else:
                    error_text = await resp.text()
                    logger.error(f"更新命名空间失败: HTTP {resp.status}, {error_text}")
                    raise ValueError(f"更新命名空间失败: HTTP {resp.status}")
        except aiohttp.ClientError as e:
            logger.error(f"网络请求失败: {e}")
            raise
        except ValueError:
            raise
        except Exception as e:
            logger.error(f"更新命名空间失败: {e}")
            raise

    async def delete_namespace(self, namespace_name: str) -> Dict[str, Any]:
        """
        删除命名空间

        Args:
            namespace_name: 命名空间名称

        Returns:
            删除结果，包含 success 和 message 字段

        Raises:
            ValueError: 删除失败（如命名空间不存在、默认命名空间不能删除等）
            aiohttp.ClientError: 网络请求失败

        示例：
            result = await client.delete_namespace("staging")
            print(result['message'])  # "命名空间已删除"

        注意:
            - 默认命名空间（default）不能删除
            - 删除操作不可逆
        """
        url = f"{self._base_api_url}/{namespace_name}/"

        logger.debug(f"删除命名空间: {namespace_name}")

        try:
            session = await self._get_session()
            headers = self._get_headers()
            async with session.delete(url, headers=headers) as resp:
                if resp.status == 200:
                    data = await resp.json()
                    logger.info(f"成功删除命名空间 '{namespace_name}'")
                    return data
                elif resp.status == 404:
                    logger.error(f"命名空间 '{namespace_name}' 不存在")
                    raise ValueError(f"命名空间 '{namespace_name}' 不存在")
                elif resp.status == 400:
                    error_text = await resp.text()
                    logger.error(f"删除命名空间失败: {error_text}")
                    raise ValueError(f"删除命名空间失败: {error_text}")
                else:
                    error_text = await resp.text()
                    logger.error(f"删除命名空间失败: HTTP {resp.status}, {error_text}")
                    raise ValueError(f"删除命名空间失败: HTTP {resp.status}")
        except aiohttp.ClientError as e:
            logger.error(f"网络请求失败: {e}")
            raise
        except ValueError:
            raise
        except Exception as e:
            logger.error(f"删除命名空间失败: {e}")
            raise

    async def close(self):
        """
        关闭 HTTP 会话，释放资源

        在不再使用客户端时应该调用此方法。
        如果使用上下文管理器（async with），会自动调用。

        示例：
            client = TaskCenterClient("http://localhost:8001")
            try:
                # 使用客户端
                namespaces = await client.get_namespace_list()
            finally:
                # 关闭客户端
                await client.close()
        """
        if self._session and not self._session.closed:
            await self._session.close()
            logger.debug("HTTP 会话已关闭")

    async def __aenter__(self):
        """支持异步上下文管理器"""
        return self

    async def __aexit__(self, _exc_type, _exc_val, _exc_tb):
        """退出上下文时自动关闭会话"""
        await self.close()

    def __repr__(self) -> str:
        """字符串表示"""
        return f"<TaskCenterClient url='{self.task_center_url}'>"




__all__ = ['TaskCenterClient']
